"""Provides Configuration class that reads the configuration files.

Loaned from instancemanager.
"""

import imp # imp exposes the python import mechanism.
import logging
import os
import os.path

log = logging.getLogger('ldap configuration')

# List renamed attributes for backwards compatibility.
#renamedAttrs = {'tgz_sources': 'archive_sources',}
renamedAttrs = {}                

class Configuration(object):
    """Takes care of reading the configuration files.

    The configuration is handled in a layered manner, with the
    possibility to overwrite the defaults of the previous
    layer. Providing sensible defaults is a key issue here.
    """

    def __init__(self):
        """Load all configuration pertaining to ldap.
        """

        self._setConfigDir()
        self.configData = {}
        self._loadConfigData()

    def _setConfigDir(self):
        """Set the config directory (instance/etc).
        """

        log.debug("Setting configuration (instance/etc) directory...")
        currentDir = os.path.dirname(os.path.abspath(__file__))
        configDir = currentDir.split(os.path.sep)[:-2]
        configDir.append('etc')
        configDir = os.path.sep.join(configDir)
        self.configDir = configDir
        log.info("Config directory is '%s'.", self.configDir)
                
    def _loadConfigData(self):
        """Recursively load configuration data.

        Sources in increasing order of preference:

        * Our own defaults (ldapconfig.py).

        * Instance defaults (ldapconfig.py in the etc dir of the instance).

        The variables imported from these files should be placed in
        self.configData.  
        """

        log.info("Loading configuration defaults.")
        import ldapconfig
        self._collectAttributes(ldapconfig)
        log.info("Overwriting configuration defaults with instance/etc.")
        instanceDefaults = self._loadInstanceConfigModule('ldapconfig')
        self._collectAttributes(instanceDefaults)
            
    def _fixupRenamedAttributes(self,attribute):
        return renamedAttrs.get(attribute, attribute)

    def _collectAttributes(self, module):
        """Collect module's attributes into self.configData.
        """

        attributes = dir(module)
        attributes = [a for a in attributes
                      if not a.startswith('_')]
        log.info("Found attributes %r.", attributes)
        tempDict = {}
        for attribute in attributes:
            dattribute = self._fixupRenamedAttributes(attribute)
            tempDict[dattribute] = getattr(module, attribute)
        self.configData.update(tempDict)
        log.debug("configData is now %r.", self.configData)

    def _loadInstanceConfigModule(self, name):
        """Return module loaded from the etc dir of the instance.
        """
        
        log.info("Trying to load instance config module '%s'.",
                  name)
        moduleFile = os.path.join(self.configDir, name+'.py')
        try:
            module = imp.load_source(name, moduleFile)
        except IOError:
            log.warn("Couldn't find '%s' in instance/etc dir.",
                     name)
            return None
        return module

